<?php

namespace ReadyApp\Taskboard\Forms;

use DialogForm;
use Application;
use Translator;
use JScript;
use WidgetException;
use DialogComposer;
use TextInput2;
use SysContext;
use ExtSelect;
use LookupWidget2;
use ContactSearchEngine;
use EDokApi;
use ContactPersonsSearchEngine;
use ContactPersonsLookupManager;
use ReadyApp\Taskboard\Services\TicketService;
use ReadyApp\Taskboard\Shared\TicketData;
use Exception;
use PgManager;

final class TicketDialog extends DialogForm {

    /**
     * Lista method do których możemy się odwołać przez JS-a
     * @var string[]
     */
    protected static $cMethods = [
        'toHtml' => 'toHtml',
    ];

    /**
     * @var TicketService
     */
    private $service;
    /**
     * @var mixed
     */
    private $dscrptToCopy;
    /**
     * @var bool
     */
    private $ticketIsHelp = FALSE;

    private $dscrptFromEmail = FALSE;

    /**
     * TicketDialog constructor.
     *
     * @param      $name
     * @param null $caption
     * @param int  $dstyle
     */
    public function __construct($name, $caption = NULL, $dstyle = BS_DIALOG) {

        parent::__construct($name, 'Ticket', BS_DIALOG);
        $this->path = Application::makeRelativePath(__FILE__);
        $this->HWND = $this->createHWND();
        $this->withoutCaption = TRUE;
        $this->width = '500px';
        $this->height = '650px';
        $this->setCaption('TICKET');

        $this->service = (new TicketService());

        if (($this->isOpened()) && (!$this->noNeedForCreate())) {
            $this->create();
        }
    }

    /**
     * @return bool[]
     */
    protected function getSupportedParams(): array {
        return ['eml_id' => TRUE,];
    }

    /**
     * @param array $params
     * @param array $data
     *
     * @return bool|void
     */
    public function open($params = NULL, &$data = NULL) {
        $data = [];

        if (!UserRights::checkSysAcc('bswfms.knowledgebase')) {
            JScript::add('alertMessage(\''.Translator::translateJS('Brak uprawnień do bazy wiedzy').'\');');
            return FALSE;
        }

        parent::open();

        $this->dscrptToCopy = $defaultData['dscrpt'];

        $this->ticketIsHelp = TRUE;
        $this->dscrptFromEmail = TRUE;
        $this->setSData(['eml_id' => $eml, 'ticketIsHelp' => $this->ticketIsHelp, 'dscrptFromEmail' => $this->dscrptFromEmail, 'optime' => $defaultData['optime']]);

        $this->create();
        $this->setWData([
            'contid' => $defaultData['contid'],
            'copeid' => $defaultData['copeid'],
        ]);

    }

    /**
     * @return bool|void
     * @throws WidgetException
     * @throws \Exception
     */
    public function save() {

        $sData = $this->getSData();
        $wData = $this->getWData();

        // Wywołanie walidacji danych
        $this->validate($wData);

        require_once('classes/eDokumentyApi/EDokApi.inc');

        $api = new EDokApi();
        $data = [
            'dscrpt' => $wData['dscrpt'],
            'dsexid' => $wData['dsexid'],
            'rspuid' => SysContext::$usr_info['usr_id'],
            'optime' => date('Y-m-d H:i:s', strtotime($sData['optime'] ?? SYSDATE)),
            'contid' => $wData['contid'],
            'pr_sta' => 1,
            'comnts' => $wData['fixinf'],
            'copeid' => $wData['copeid'],
        ];

        try {
            $prc_id = $api->createProcess($data);

        } catch (Exception $e) {
            throw new Exception(Translator::translate('Nie udało się założyć sprawy!'));
        }
        try {
            if ($wData['ftcomp']) {
                $api->setFeatureValue($this->service->getFeatureID(TicketData::COMPONENT), 'processes', $prc_id, $wData['ftcomp']);
            }
            if ($wData['ftprior']) {
                $api->setFeatureValue($this->service->getFeatureID(TicketData::SLA), 'processes', $prc_id, $wData['ftprior']);
            }
            if ($wData['ftenv']) {
                $api->setFeatureValue($this->service->getFeatureID(TicketData::ENVIRONMENT), 'processes', $prc_id, $wData['ftenv']);
            }
            if ($wData['ftvers']) {
                $api->setFeatureValue($this->service->getFeatureID(TicketData::VERSION), 'processes', $prc_id, $wData['ftvers']);
            }
        } catch (Exception $e) {

            $db = PgManager::getInstance();
            if (!$db) {
                return;
            }
            $db->rollback($db->transactionInProgress);
            throw new WidgetException(NULL, $e->getMessage());
        }

        if (isset($sData['dscrptFromEmail'])) {
            $doc_id = $this->service->getMailArchiveDocumentId($this->service->getMailBean($sData['eml_id']));
            $api->updateDocument(['doc_id' => $doc_id, 'prc_id' => $prc_id]);
        }
        JScript::addAfter('App.openDialogByCls(\'PROCESS\', \''.$prc_id.'\').toJSONString();');

        $this->jsClose(TRUE);
    }

    /**
     * @return false|void
     */
    protected function create() {
    	/**
    	* Sprawdzamy czy dialog został już wcześniej otworzony
    	*/
        if (!$this->isOpened()) {
            return FALSE;
        }
        $data = $this->getSData();

        $composer = new DialogComposer($this);
        $composer->addModernHeader(Translator::translate('Utwórz nowy ticket'), 'eic eic-e-service', '#2c3e50');

        $this->dscrpt = new TextInput2($this->name.'dscrpt');
        $this->dscrpt->setLabel(Translator::translate('Opis'));
        if ($this->dscrptFromEmail && $this->ticketIsHelp) {
            $this->dscrpt->labelWidget->width = 'calc(100% - '.(\Dialog::SPACE_LEFT + \Dialog::SPACE_RIGHT).'px)';
            $this->dscrpt->labelWidget->text .= ('<a class="ActionLink" onclick ="document.getElementById(\''.$this->dscrpt->getName().'\').innerHTML = \''.$this->dscrptToCopy.'\';" style="position:absolute; display:inline-block; right:0px;">Wstaw z maila</a>');
            $this->dscrpt->getName();
        }
        $this->dscrpt->height = (2 * self::DEFAULT_LINE_HEIGHT).'px';
        $this->dscrpt->setRequired(TRUE);

        $composer->addNext($this->dscrpt, 'dscrpt');

        /* Typ / dział do wyboru - teczka */
        $this->dsexid = new ExtSelect($this->name.'dsexid');
        $this->dsexid->withTitles = TRUE;
        $this->dsexid->cutTitles = FALSE;
        $this->dsexid->width = 'calc(50% - '.(1.5 * self::SPACE_LEFT).'px)';
        $this->dsexid->setLabel(Translator::translate('Dział/Typ'));
        $this->dsexid->setRequired(TRUE);

        if (isset(SysContext::$usr_info['orunid'][0])) {

            $this->dsexid->setData(NULL);
            $places = ($this->ticketIsHelp) ? $this->service->brifcaseData(['\''.TicketData::PROCESS_HELPDESK.'\'']) : $this->service->brifcaseData(['\''.TicketData::PROCESS_ENHANCEMENT.'\',\''.TicketData::PROCESS_FEATURE.'\',\''.TicketData::PROCESS_DEFECT.'\', \''.TicketData::PROCESS_DEVOPS.'\'']);

            foreach ($places as $v) {
                $item = [];
                $item['value'] = $v['dsexid'];
                $item['caption'] = $v['text'];

                if (isset($v['fxtrid']) and $v['fxtrid']) {
                    $item['data'] = ['fxtrvl' => $v['fxtrvl']];
                }

                if (isset($v['title'])) {
                    $item['title'] = $v['title'];
                }
                $this->dsexid->addItem($v['dsexid'], $item, ($v['grp'] ?? NULL));
            }
        }

        $composer->addNext($this->dsexid, 'dsexid');

        /* Kategoria SLA */
        $this->ft_prior = new ExtSelect($this->name.'ftprior');
        $this->ft_prior->setLabel(Translator::translate('Kategoria SLA'));
        $this->ft_prior->setRequired(FALSE);
        $this->ft_prior->query = 'SELECT sla_problem_cat AS value, name AS caption FROM cregisters.creg_sla_categories WHERE NOT is_del';
        $this->ft_prior->update();
        $this->ft_prior->width = 'calc(50% - '.(1.5 * self::SPACE_LEFT).'px)';

        $composer->addNext($this->ft_prior, 'ftprior', self::P_TO_RIGHT);

        /* Środowisko */
        $this->ftenv = new ExtSelect($this->name.'ftenv');
        $this->ftenv->query = 'SELECT fo.ftopid  AS value, fod.ftopnm AS caption FROM features f 
        INNER JOIN features_options fo ON fo.featid = f.featid INNER JOIN features_options_def fod ON fod.fodfid = fo.fodfid 
        WHERE f.ftsmbo=\''.TicketData::ENVIRONMENT.'\' AND NOT is_del ORDER BY fod.ftopnm';
        $this->ftenv->update();
        $this->ftenv->setLabel(Translator::translate('Środowisko'));
        $this->ftenv->setRequired();
        $this->ftenv->width = 'calc(50% - '.(1.5 * self::SPACE_LEFT).'px)';
        $composer->addNext($this->ftenv, 'ftenv', self::P_TO_LEFT);

        /* Komponent / aplikacja */
        $this->ftcomp = new ExtSelect($this->name.'ftcomp');
        $this->ftcomp->query = 'SELECT app_id AS value, appnam AS caption FROM cregisters.creg_applications WHERE NOT is_del ORDER BY app_id';
        $this->ftcomp->update();
        $this->ftcomp->setLabel(Translator::translate('Komponent'));
        $this->ftcomp->setRequired();
        $this->ftcomp->width = 'calc(50% - '.(1.5 * self::SPACE_LEFT).'px)';
        $composer->addNext($this->ftcomp, 'ftcomp', self::P_TO_LEFT);

        /* Wersja komponentu */
        $this->ftvers = new TextInput2($this->name.'ftvers');
        $this->ftvers->setLabel(Translator::translate('Wersja komponentu'));
        $this->ftvers->setRequired(FALSE);
        $this->ftvers->right = 'calc( 22% )';
        $this->ftvers->width = 'calc( 30% - '.(1 * self::SPACE_LEFT).'px)';
        $composer->addNext($this->ftvers, 'ftvers', self::P_TO_RIGHT);

        /* Klient kopiowany z maila */
        $this->contid = new LookupWidget2($this->name.'contid', new ContactSearchEngine(), FALSE, 1);
        $this->contid->setLabel(Translator::translate('Klient'));
        $this->contid->width = 'calc( 100% - '.(2 * DialogForm::SPACE_LEFT).'px )';
        $composer->addNext($this->contid, 'contid', DialogForm::P_TO_LEFT);

        /* Osoba kontaktowa z maila */
        $this->copeid = new LookupWidget2($this->name.'_copeid', new ContactPersonsSearchEngine(), FALSE, TRUE);
        $this->copeid->setFilterString('defcnt IS NOT NULL');
        $this->copeid->setLabel(Translator::translate('Osoba kontaktowa'));
        require_once('./classes/LookupWidget/ContactPersons/ContactPersonsLookupManager.inc');
        ContactPersonsLookupManager::manage($this->copeid, ['contid' => $this->contid->name]);
        $composer->addNext($this->copeid, 'copeid');

        /* Uwagi */
        $this->fixinf = new TextInput2($this->name.'fixinf');
        $this->fixinf->setLabel(Translator::translate('Uwagi'));
        $this->fixinf->setRequired(FALSE);
        $this->fixinf->height = ((2 * self::DEFAULT_LINE_HEIGHT) + (self::DEFAULT_WIDGET_HEIGHT * 2)).'px';
        $this->fixinf->width = 'calc( 100% - '.(2 * self::SPACE_LEFT).'px)';
        $composer->addNext($this->fixinf, 'fixinf', self::P_TO_LEFT);

        /**
        * Dodanie przycisków Zapisu i Anuluj
        */
        $composer->addModernButtonsPanel([
            'bSave',
            'bCancel',
        ]);
        $this->add($composer);

    }

    /**
     * Walidaca danych z formularza
     * @param array $data
     *
     * @return void
     */
    private function validate(array $data) {

        if (bs_empty_str($data['dscrpt'])) {
            throw new WidgetException($this->dscrpt->getName(), Translator::translate('Uzupełnij opis'));
        }
        if ($data['dsexid'] === NULL || $data['dsexid'] < 1) {
            throw new WidgetException($this->dsexid->getName(), Translator::translate('Wybierz dział/typ'));
        }

        if ($data['ftcomp'] === NULL || $data['ftcomp'] < 1) {
            throw new WidgetException($this->ftcomp->getName(), Translator::translate('Wybierz komponent'));
        }

    }

}
